from .configure import CertificateSource as CertificateSource, PrivateKeySource as PrivateKeySource, add_ca_certificate as add_ca_certificate, add_client_certificate as add_client_certificate
from .encodings import AsciiArmored as AsciiArmored, Certificate as Certificate, ECPrivateKey as ECPrivateKey, Pkcs12 as Pkcs12, Pkcs8EncryptedPrivateKey as Pkcs8EncryptedPrivateKey, Pkcs8PrivateKey as Pkcs8PrivateKey, PrivateKey as PrivateKey, RSAPrivateKey as RSAPrivateKey
from .files import EncodedFile as EncodedFile

__all__ = ['AsciiArmored', 'Certificate', 'CertificateSource', 'ECPrivateKey', 'EncodedFile', 'Pkcs8EncryptedPrivateKey', 'Pkcs8PrivateKey', 'Pkcs12', 'PrivateKey', 'PrivateKeySource', 'RSAPrivateKey', 'add_ca_certificate', 'add_client_certificate']



# Copyright 2023, 2025  Dom Sekotill <dom.sekotill@kodo.org.uk>

from ._enums import MILLISECONDS
from ._enums import SECONDS
from ._enums import Time
from ._exceptions import CurlError
from ._multi import Multi
from .requests import Request

__all__ = [
	"MILLISECONDS",
	"SECONDS",
	"CurlError",
	"Multi",
	"Request",
	"Time",
]



from .encodings import AsciiArmored, Certificate, Pkcs12, PrivateKey
from .files import EncodedFile
from pathlib import Path

__all__ = ['identify_blob', 'identify_certificate_file', 'identify_file']

def identify_file(path: Path) -> EncodedFile[AsciiArmored] | EncodedFile[Certificate] | EncodedFile[Pkcs12] | EncodedFile[PrivateKey]: ...
def identify_certificate_file(path: Path) -> EncodedFile[AsciiArmored] | EncodedFile[Certificate] | EncodedFile[Pkcs12]: ...
def identify_blob(blob: bytes) -> AsciiArmored | Certificate | Pkcs12 | PrivateKey: ...



from ..abc import ConfigHandle
from .encodings import AsciiArmored, Certificate, Encoding, Pkcs12, PrivateKey
from .files import EncodedFile
from pathlib import Path
from typing import TypeAlias, TypeVar, overload

__all__ = ['CertificateSource', 'PrivateKeySource', 'add_ca_certificate', 'add_client_certificate']

ContainerT = TypeVar('ContainerT', AsciiArmored, Pkcs12)
EncodedT = TypeVar('EncodedT', bound=Encoding)
RawT = TypeVar('RawT', Certificate, PrivateKey)
CommonEncodedSource: TypeAlias = AsciiArmored | Pkcs12 | EncodedFile[AsciiArmored] | EncodedFile[Pkcs12]
EncodedSource: TypeAlias = CommonEncodedSource | RawT | EncodedFile[RawT]
CertificateSource: TypeAlias = EncodedSource[Certificate]
PrivateKeySource: TypeAlias = EncodedSource[PrivateKey]

def add_ca_certificate(handle: ConfigHandle, cert_source: CertificateSource | Path) -> None: ...
@overload
def add_client_certificate(handle: ConfigHandle, cert: CertificateSource, key: PrivateKeySource) -> None: ...
@overload
def add_client_certificate(handle: ConfigHandle, cert: CommonEncodedSource, key: None = None) -> None: ...



from .encodings import Encoding as Encoding
from _typeshed import Incomplete
from pathlib import Path
from typing import Final, Generic, TypeVar

T = TypeVar('T', bound=Encoding, covariant=True)
C = TypeVar('C', bound=Encoding)
DEFAULT_MAX_READ: Final[Incomplete]

class EncodedFile(Generic[T]):
    contents: Incomplete
    path: Incomplete
    def __init__(self, contents: T, path: Path) -> None: ...
    @classmethod
    def read(cls, path: Path, encoding: type[C], /, maxsize: int = ...) -> EncodedFile[C]: ...
    @staticmethod
    def write(path: Path, contents: C, /, *, exists_ok: bool = False) -> EncodedFile[C]: ...



from _typeshed import Incomplete
from collections.abc import Iterator, Sequence
from typing import Literal, Self, SupportsIndex, TypeAlias, overload

BytesType: TypeAlias = Sequence[SupportsIndex]
Header: TypeAlias = tuple[str, str]
CHR_EQL: Incomplete
CHR_DASH: Incomplete

class ArmoredData(bytes):
    def __new__(cls, label: str, source: BytesType, headers: Sequence[Header] | None = None) -> Self: ...
    label: Incomplete
    headers: Sequence[Header]
    def __init__(self, label: str, source: BytesType, headers: Sequence[Header] | None = None) -> None: ...
    def encode_lines(self) -> Iterator[bytes]: ...
    @overload
    @classmethod
    def extract(cls, data: bytes | bytearray, *, with_unarmored: Literal[True]) -> Iterator[Self | str]: ...
    @overload
    @classmethod
    def extract(cls, data: bytes | bytearray, *, with_unarmored: Literal[False] = False) -> Iterator[Self]: ...



from typing import ClassVar, Protocol, Self, TypeVar, overload

__all__ = ['AsciiArmored', 'Certificate', 'ECPrivateKey', 'Pkcs8EncryptedPrivateKey', 'Pkcs8PrivateKey', 'Pkcs12', 'PrivateKey', 'RSAPrivateKey']

PrivateKeyT = TypeVar('PrivateKeyT', bound=PrivateKey)

class Encoding(Protocol):
    format: ClassVar[str]
    @classmethod
    def from_bytes(cls, source: bytes, /) -> Self: ...
    def to_bytes(self) -> bytes: ...
    def certificate(self) -> Certificate | None: ...
    def private_key(self) -> PrivateKey | None: ...

class Certificate(bytes):
    format: ClassVar
    label: ClassVar
    @classmethod
    def from_bytes(cls, source: bytes, /) -> Self: ...
    def to_bytes(self) -> Self: ...
    def fingerprint(self) -> str: ...
    def certificate(self) -> Self: ...
    def private_key(self) -> None: ...

class PrivateKey(bytes):
    format: ClassVar[str]
    label: ClassVar[str]
    def __new__(cls, source: bytes, /) -> Self: ...
    def __init_subclass__(cls, label: str) -> None: ...
    @classmethod
    def from_bytes(cls, source: bytes, /) -> Self: ...
    def to_bytes(self) -> Self: ...
    def certificate(self) -> None: ...
    def private_key(self) -> Self: ...

class RSAPrivateKey(PrivateKey, label='RSA PRIVATE KEY'): ...
class Pkcs8PrivateKey(PrivateKey, label='PRIVATE KEY'): ...
class Pkcs8EncryptedPrivateKey(PrivateKey, label='ENCRYPTED PRIVATE KEY'): ...
class ECPrivateKey(PrivateKey, label='EC PRIVATE KEY'): ...

class AsciiArmored(bytes):
    format: ClassVar
    @classmethod
    def new(cls, certificate: Certificate | None = None, private_key: PrivateKey | None = None) -> Self: ...
    @classmethod
    def from_bytes(cls, source: bytes, /) -> Self: ...
    def to_bytes(self) -> Self: ...
    def certificate(self) -> Certificate | None: ...
    def private_key(self) -> PrivateKey | None: ...
    @overload
    def find_first(self, kind: type[Certificate], /) -> Certificate: ...
    @overload
    def find_first(self, kind: type[PrivateKeyT], /) -> PrivateKeyT: ...

class Pkcs12(bytes):
    format: ClassVar
    @classmethod
    def new(cls, certificate: Certificate | None = None, private_key: PrivateKey | None = None) -> Self: ...
    @classmethod
    def from_bytes(cls, source: bytes, /) -> Self: ...
    def to_bytes(self) -> Self: ...
    def certificate(self) -> Certificate | None: ...
    def private_key(self) -> PrivateKey | None: ...
